unit FormBIGMain;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList, StdActns,
  ActnList, ToolWin, BasicDataTypes, BIG_File, FormDestination, SysUtils,
  BasicFunctions;

const
   APPLICATION_NAME = 'OS BIG Editor';
   APPLICATION_VERSION = '0.4 Beta';
   APPLICATION_AUTHORs = 'Banshee';

type
  TFrmBIGMain = class(TForm)
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    ToolBar1: TToolBar;
    TbFileNew: TToolButton;
    ToolButton1: TToolButton;
    TbFileSave: TToolButton;
    ToolButton3: TToolButton;
    TbEditCut: TToolButton;
    TbEditCopy: TToolButton;
    TbEditPaste: TToolButton;
    ActionList1: TActionList;
    FileNew1: TAction;
    FileOpen1: TAction;
    FileSave1: TAction;
    FileSaveAs1: TAction;
    FileExit1: TAction;
    EditCut1: TEditCut;
    EditCopy1: TEditCopy;
    EditPaste1: TEditPaste;
    HelpAbout1: TAction;
    StatusBar: TStatusBar;
    ImageList1: TImageList;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    Edit1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    Help1: TMenuItem;
    HelpAboutItem: TMenuItem;
    N2: TMenuItem;
    ExtractItem: TMenuItem;
    ExtractAllItem: TMenuItem;
    LbFiles: TListBox;
    SpContent: TSplitter;
    MmFileContent: TMemo;
    EditExtract: TAction;
    EditExtractAll: TAction;
    Options1: TMenuItem;
    WordWrapItem: TMenuItem;
    procedure WordWrapItemClick(Sender: TObject);
    procedure EditExtractAllExecute(Sender: TObject);
    procedure EditExtractExecute(Sender: TObject);
    // Constructors and Destructors.
    procedure FormCreate(Sender: TObject);
    // I/O
    procedure FileNew1Execute(Sender: TObject);
    procedure FileOpen1Execute(Sender: TObject);
    procedure FileSave1Execute(Sender: TObject);
    // Gets
    // Sets
    procedure SetIsEditable(Value : Boolean);
    // Interface Events
    procedure SpContentMoved(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure LbFilesClick(Sender: TObject);
    procedure FileExit1Execute(Sender: TObject);
    procedure HelpAbout1Execute(Sender: TObject);

  private
    { Private declarations }
    IsEditable : Boolean;
    // I/O
    procedure LoadBIGFile(const _Filename : string);
    // Gets
    Function GetParamStr : String;
  public
    { Public declarations }
    CurrentFile : TBIGPackage;
  end;

var
  FrmBIGMain: TFrmBIGMain;

implementation

uses FormAboutBox;

{$R *.dfm}

// Constructors and Destructors.
procedure TFrmBIGMain.FormCreate(Sender: TObject);
var
   MyParameter : string;
begin
   CurrentFile := TBIGPackage.Create;
   Caption := APPLICATION_NAME + ' ' + APPLICATION_VERSION;
   SetIsEditable(false);
   StatusBar.Panels.Add;

   // Quick load file through parameter;
   if ParamCount > 0 then
   begin
      MyParameter := GetParamStr;
      If FileExists(MyParameter) then
      Begin
         LoadBIGFile(MyParameter);
      end;
   End;
end;

// I/O
procedure TFrmBIGMain.FileNew1Execute(Sender: TObject);
begin
  { Do nothing }
end;

procedure TFrmBIGMain.LoadBIGFile(const _Filename : string);
var
   I : uint32;
   iFile : TBIGFileUnit;
begin
   SetIsEditable(false);
   LbFiles.Clear;
   MmFileContent.Clear;
   CurrentFile.LoadFile(_FileName);
   if CurrentFile.IsValid then
   begin
      for I := 0 to (CurrentFile.GetNumFiles-1) do
      begin
         iFile := CurrentFile.GetFileInfo(i);
         LbFiles.Items.Add(iFile.Filename);
      end;
      lbFiles.ItemIndex := 0;
      LbFilesClick(nil);
      SetIsEditable(true);
   end;
end;

procedure TFrmBIGMain.FileOpen1Execute(Sender: TObject);
begin
   if OpenDialog.Execute then
   begin
      LoadBIGFile(OpenDialog.FileName);
   end;
end;

procedure TFrmBIGMain.FileSave1Execute(Sender: TObject);
begin
   SaveDialog.Execute;
end;

// Gets

// Ripped from Voxel Section Editor III 1.36 and all other OS Tools.
// Probably writen by Stucuk.
Function TFrmBIGMain.GetParamStr : String;
var
   x : integer;
begin
   Result := '';
   for x := 1 to ParamCount do
      if Result <> '' then
         Result := Result + ' ' + ParamStr(x)
      else
         Result := ParamStr(x);
end;


// Sets
procedure TFrmBIGMain.SetIsEditable(Value : Boolean);
begin
   IsEditable := Value;
   FileNewItem.Enabled := false;
   TbFileNew.Enabled := false;
   FileSaveItem.Enabled := Value;
   FileSaveAsItem.Enabled := Value;
   TbFileSave.Enabled := Value;
   TbEditPaste.Enabled := false;
   PasteItem.Enabled := false;
   TbEditCopy.Enabled := false;
   CopyItem.Enabled := false;
   TbEditCut.Enabled := false;
   CutItem.Enabled := false;
   ExtractItem.Enabled := Value;
   ExtractAllItem.Enabled := Value;
   Options1.Visible := false;
   WordWrapItem.Enabled := false;
end;

// Interface Events
procedure TFrmBIGMain.FileExit1Execute(Sender: TObject);
begin
  Close;
end;

procedure TFrmBIGMain.HelpAbout1Execute(Sender: TObject);
begin
  FrmAboutBox.ShowModal;
end;

procedure TFrmBIGMain.LbFilesClick(Sender: TObject);
var
   Data : TStream;
   DataSize : uint32;
   Index : uint32;
   MyFileInfo : TBIGFileUnit;
begin
   Index := lbFiles.ItemIndex;
   MyFileInfo := CurrentFile.GetFileInfo(Index);
   CurrentFile.GetFileContents(Index,Data,DataSize);
   MmFileContent.Clear;
   Data.Seek(0,soFromBeginning);
   mmFileContent.Lines.LoadFromStream(Data); //.Add(PData.ReadString(DataSize));
   lbFiles.Hint := MyFileInfo.Filename;
   if DataSize = MyFileInfo.Size then
      {StatusBar.Panels[0].Text}lbFiles.Hint := (MyFileInfo.Filename +  ' - ' + IntToStr(DataSize) + ' bytes')
   else
      {StatusBar.Panels[0].Text}lbFiles.Hint := (MyFileInfo.Filename +  ' - ' + IntToStr(MyFileInfo.Size) + ' bytes compressed ' + ' - ' + IntToStr(DataSize) + ' bytes uncompressed (' + FloatToStrF((MyFileInfo.Size / DataSize) * 100,ffGeneral,4,4) + '%)');
   Data.Free;
end;

procedure TFrmBIGMain.FormResize(Sender: TObject);
begin
   lbFiles.Left := 0;
   lbFiles.Width := SpContent.Left - lbFiles.Left - 1;
   mmFileContent.Left := spContent.Left + 1;
   mmFileContent.Width := Width - mmFileContent.Left - 5;
end;

procedure TFrmBIGMain.SpContentMoved(Sender: TObject);
begin
   FormResize(sender);
end;

procedure TFrmBIGMain.EditExtractExecute(Sender: TObject);
var
   Data : TStream;
   DataSize : uint32;
   Index : uint32;
   MyFileInfo : TBIGFileUnit;
   MyFile : TStream;
   MyFileName : string;
   MyDirectory : string;
   Form : TFrmDestination;
begin
   if lbFiles.SelCount > 0 then
   begin
      Form := TFrmDestination.Create(self);
      Form.ShowModal;
      if Form.Changed then
      begin
         for Index := 0 to CurrentFile.GetNumFiles-1 do
         begin
            if lbFiles.Selected[Index] then
            begin
               MyFileInfo := CurrentFile.GetFileInfo(Index);
               CurrentFile.GetFileContents(Index,Data,DataSize);
               Data.Seek(0,soFromBeginning);
               MyFilename := Form.Directory.Path + '\' + MyFileInfo.Filename;
               MyDirectory := ExtractFileDir(MyFileName);
               if not DirectoryExists(MyDirectory) then
                  MakeMeADir(MyDirectory);
               MyFile := TFileStream.Create(MyFileName,fmCreate);
               MyFile.Seek(0,soFromBeginning);
               MyFile.CopyFrom(Data,DataSize);
               MyFile.Free;
               Data.Free;
            end;
         end;
      end;
   end
   else
   begin
      Index := LbFiles.ItemIndex;
      MyFileInfo := CurrentFile.GetFileInfo(Index);
      SaveDialog.FileName := MyFileInfo.Filename;
      if SaveDialog.Execute() then
      begin
         CurrentFile.GetFileContents(Index,Data,DataSize);
         Data.Seek(0,soFromBeginning);
         MyDirectory := ExtractFileDir(SaveDialog.FileName);
         if not DirectoryExists(MyDirectory) then
            MakeMeADir(MyDirectory);
         MyFile := TFileStream.Create(SaveDialog.FileName,fmCreate);
         MyFile.Seek(0,soFromBeginning);
         MyFile.CopyFrom(Data,DataSize);
         MyFile.Free;
         Data.Free;
      end;
   end;
end;

procedure TFrmBIGMain.EditExtractAllExecute(Sender: TObject);
var
   Data : TStream;
   DataSize : uint32;
   Index : uint32;
   MyFileInfo : TBIGFileUnit;
   MyFile : TStream;
   Form : TFrmDestination;
   MyFileName : string;
   MyDirectory : string;
begin
   Form := TFrmDestination.Create(self);
   Form.ShowModal;
   if Form.Changed then
   begin
      for Index := 0 to CurrentFile.GetNumFiles-1 do
      begin
         MyFileInfo := CurrentFile.GetFileInfo(Index);
         CurrentFile.GetFileContents(Index,Data,DataSize);
         Data.Seek(0,soFromBeginning);
         MyFilename := Form.Directory.Path + '\' + MyFileInfo.Filename;
         MyDirectory := ExtractFileDir(MyFilename);
         if not DirectoryExists(MyDirectory) then
            MakeMeADir(MyDirectory);
         MyFile := TFileStream.Create(MyFilename,fmCreate{and fmOpenWrite});
         MyFile.Seek(0,soFromBeginning);
         MyFile.CopyFrom(Data,DataSize);
         MyFile.Free;
         Data.Free;
      end;
   end;
   Form.Release;
end;

procedure TFrmBIGMain.WordWrapItemClick(Sender: TObject);
begin
   WordWrapItem.Checked := not WordWrapItem.Checked;
   mmFileContent.WordWrap := WordWrapItem.Checked;
end;

end.
